//------------------------------------------------------------
/**
 * Camera stocke les coordonnes de la camra
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder 
 * @date        janvier 2004
 * @version     0.22
 */
public class Camera {
  public static final double VITESSE_ROTX = 1.0 ;
  public static final double VITESSE_ROTY = 1.0 ;

  private static final double VITESSE_MAX = 10.0 ;
  private static final double ACCELERATION_MAX = 1.0 ;

  private static final double YMIN = 1.0 ;
  
  private double x,y,z ;
  private double x2,y2,z2 ;
  private double rotx,roty ;
  private double vitesse ;

  /** 
   * numero de la fourmi suivie en mode tracking
   */
  private int trackingFourmi=-1;  

  /** 
   * numero de la fourmiliere en mode tracking
   */
  private int trackingFourmiliere=-1; 

  //------------------------------------------------------------
  /** 
   * Constructeur par dfaut
   */
  public Camera(double mapx,double mapy) {
    vitesse = 0 ;

    // Centre la camera
    x=mapx/2;
    z=mapy/2;

    // Altitude initiale
    y=100;  

    rotx = roty = 0 ;

    calculeDirection() ;

  }

  public void affiche() {
    System.out.println ("("+x+","+y+","+z+")->("+x2+","+y2+","+z2+") # rot("+rotx+","+roty+")") ;
  }

  //------------------------------------------------------------
  /** 
   * Modificateur x, y, z
   */
  public void setXYZ(int x0, int y0, int z0) {
    x = x0 ; y = y0 ; z = z0 ;
    calculeDirection() ;
  }
    
  public void setXY(int x0, int y0) {
    x = x0 ; y = y0 ;
    calculeDirection() ;
  }
    
  //------------------------------------------------------------
  /** 
   * Incrementeur x, y, z
   */
  public void incXYZ(int x0, int y0, int z0) {
    x += x0 ; y += y0 ; z += z0 ;
    if (y<YMIN) y=YMIN ;
    calculeDirection() ;
  }
    
  //------------------------------------------------------------
  /** 
   * Modificateur x, y, z
   */
  public void incRotXY(double incx, double incy) {
    rotx += incx ; roty += incy ;
    calculeDirection() ;
  }
    
  //------------------------------------------------------------
  /** 
   * Acclration de la camra
   */
  public void incVitesse() {
    vitesse += ACCELERATION_MAX ;
    if (vitesse>VITESSE_MAX) vitesse = VITESSE_MAX ;
  }
    
  //------------------------------------------------------------
  /** 
   * Dclration de la camra
   */
  public void decVitesse() {
    vitesse -= ACCELERATION_MAX ;
    if (vitesse<0) vitesse = 0 ;
  }
    
  //------------------------------------------------------------
  /** 
   * Init du vol de la camra
   */
  public void initFly() {
    vitesse = 0.0 ;
  }
    
  //------------------------------------------------------------
  /** 
   * Calcule un point vis par la camra en fonction
   * de sa position et de ses angles
   */
  public void calculeDirection() {
    double anglex = getRotX()*Simulation.CONV_DEG_RAD ;
    double angley = getRotY()*Simulation.CONV_DEG_RAD ;

    double cosanglex = Math.cos(anglex) ;

    x2 = x+100.0*Math.cos(angley)*cosanglex ;
    y2 = y+100.0*Math.sin(anglex) ;
    z2 = z+100.0*Math.sin(angley)*cosanglex ;

    //    affiche() ;
  }

  //------------------------------------------------------------
  public void translateDevant() {
    double oldvitesse = vitesse ;
    vitesse = VITESSE_MAX ;
    fly() ;
    vitesse = oldvitesse ; 
  }

  //------------------------------------------------------------
  public void translateDerriere() {
    double oldvitesse = vitesse ;
    vitesse = -VITESSE_MAX ;
    fly() ;
    vitesse = oldvitesse ; 
  }

  //------------------------------------------------------------
  public void translateDroite() {
    double angley = getRotY()*Simulation.CONV_DEG_RAD ;
    incXYZ ((int)(-VITESSE_MAX*Math.sin(angley)), 0,
            (int)(VITESSE_MAX*Math.cos(angley)));
  }

  //------------------------------------------------------------
  public void translateGauche() {
    double angley = getRotY()*Simulation.CONV_DEG_RAD ;
    incXYZ ((int)(VITESSE_MAX*Math.sin(angley)), 0,
            (int)(-VITESSE_MAX*Math.cos(angley)));
  }

  //------------------------------------------------------------
  public void translateHaut() {
    incXYZ(0, (int)VITESSE_MAX, 0) ;
  }

  //------------------------------------------------------------
  public void translateBas() {
    incXYZ(0, -(int)VITESSE_MAX, 0) ;
  }

  //------------------------------------------------------------
  /** 
   * Vol de la camra
   */
  public void fly() {
    double anglex = getRotX()*Simulation.CONV_DEG_RAD ;
    double angley = getRotY()*Simulation.CONV_DEG_RAD ;

    double cosanglex = Math.cos(anglex) ;

    incXYZ ((int)(vitesse*Math.cos(angley)*cosanglex),
            (int)(vitesse*Math.sin(anglex)),
            (int)(vitesse*Math.sin(angley)*cosanglex));
  }
    
  //------------------------------------------------------------
  /** 
   * Accesseurs
   */
  public double getX() { return x ; }
  public double getY() { return y ; }
  public double getZ() { return z ; }
  public double getX2() { return x2 ; }
  public double getY2() { return y2 ; }
  public double getZ2() { return z2 ; }
  public double getRotX() { return rotx ; }
  public double getRotY() { return roty ; }
}
