//------------------------------------------------------------
/**
 * Objet 
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder
 * @date        janvier 2004
 * @version     0.22
 */

   import java.awt.* ;

    abstract public class Objet {
    
      public static final int OBSTACLE = 1 ;
      public static final int NOURRITURE = 2 ;
      public static final int FOURMILIERE = 3 ;
      public static final int FOURMI = 4 ;
      public static final int PHEROMONE = 5 ;
   
      public static final float[] RED=  {0, (float)1.0,  0,  0,(float)1.0,(float)1.0,(float)1.0};
      public static final float[] GREEN={0,   0,(float)1.0,(float)1.0,(float)1.0,  0,(float)1.0} ;
      public static final float[] BLUE= {(float)1.0, 0,  0,(float)1.0,  0,(float)1.0,(float)1.0} ;
   
      protected int type ;
      protected int id, idGroup ;
      protected boolean objetStatic ;
   
      protected double x, y, z ; 
      protected double roty ;
      protected double scale ;
   
      protected Polygon polygon, polygonZoom ;
   
   // withColor=true : prendre la couleur dfinie dans le fichier
   // withColor=false : prendre les couleurs red, green, blue
      protected boolean withColor ; 
      protected float red, green, blue ;
      protected Color color ;
   
   //------------------------------------------------------------
   /** 
   * Constructeur par dfaut
   */
       public Objet(){
         x= y= z= 0.0 ;
         roty = 0.0 ;
         scale = 1.0 ;
         type = 0 ;
         id = 0 ;
         idGroup = 0 ;
         objetStatic = true ;
         setColor((float)1.0, (float)1.0, (float)1.0) ;
      }
   
   //------------------------------------------------------------
       public Objet(double x0, double y0){
         x = x0 ; y = y0 ; 
         z = 0 ; 
         roty = 0.0 ;
         scale = 1.0 ;
         type = 0 ;
         id = 0 ;
         idGroup = 0 ;
         objetStatic = true ;
         setColor((float)1.0, (float)1.0, (float)1.0) ;
      }
   
   //------------------------------------------------------------
       public Objet(double x0, double y0, int id0, int idg0){
         x = x0 ; y = y0 ; 
         z = 0 ; 
         roty = 0.0 ;
         scale = 1.0 ;
         id = id0 ;
         idGroup = idg0 ;
         setColor((float)1.0, (float)1.0, (float)1.0) ;
      }
   
   //------------------------------------------------------------
       public Objet(double x0, double y0, double ry, double s, int id0, int idg0){
         x = x0 ; y = y0 ; roty = ry ; scale = s ;
         z = 0 ; 
         id = id0 ;
         idGroup = idg0 ;
         withColor = true ;
         setColor((float)1.0, (float)1.0, (float)1.0) ;
      }
   
   //------------------------------------------------------------
       public Objet(double x0, double y0, double z0, double ry, double s, int id0, int idg0){
         x = x0 ; y = y0 ; z = z0 ; roty = ry ; scale = s ;
         id = id0 ;
         idGroup = idg0 ;
         withColor = true ;
         setColor((float)1.0, (float)1.0, (float)1.0) ;
      }
   
   //------------------------------------------------------------
   /** 
   * Accesseurs
   */
       public double getX() { 
         return x ; }
       public double getY() { 
         return y ; }
       public double getZ() { 
         return z ; }
       public double getRot() { 
         return roty ; }
       public double getScale() { 
         return scale ; }
       public int getType() { 
         return type ;}
       public int getId() { 
         return id ; }
       public int getIdGroup() { 
         return idGroup ; }
       public boolean isWithColor() { 
         return withColor ; }
       public float getRed() { 
         return red ; }
       public float getGreen() { 
         return green ; }
       public float getBlue() { 
         return blue ; }
       public boolean isStatic() { 
         return objetStatic ; }
       public Color getColor() { 
         return color ;}
       public int getContenu() { 
         return 0 ; }
   
   //------------------------------------------------------------
   /** 
   * simulation de l'objet
   */
       public void run() {
      }
   
   //------------------------------------------------------------
       public boolean contains(double x, double y) {
         if (polygon!=null) 
            return polygon.contains(x,y) ;
         return false ;
      }
   
   //------------------------------------------------------------
   /** 
   * Modification du type
   */
       public void setType(int t){
         type = t ;
         objetStatic = true ;
         switch (type) {
            case FOURMI :
               objetStatic = false ;
               break ;
         }
      }
   
   //------------------------------------------------------------
   /** 
   * Modification de la couleur
   */
       public void setColor(float r0, float g0, float b0){
         withColor = false ;
         red = r0 ; green = g0 ; blue = b0 ;
         color = new Color(red,green,blue) ;
      }
   
   //------------------------------------------------------------
   /** 
   * Convertit un objet en String
   */
       public String toString() {
         String st = new String() ;
         switch (type) {
            case OBSTACLE : st = st+"Obstacle " ; 
               break ;
            case NOURRITURE : st = st+"Nourriture " ; 
               break ;
            case FOURMILIERE : st = st+"Fourmiliere " ; 
               break ;
            case FOURMI : st = st+"Fourmi " ; 
               break ;
            case PHEROMONE : st = st+"Pheromone " ; 
               break ;
            default : st = st+"Type " ; 
               break ;
         }
         st = st + id + "/"+idGroup+":("+x+","+y+","+z+") ; rot="+roty+" ; scale = "+scale ;
         st = st + " ; static="+objetStatic ;
         return st ;
      }
   
   
   }

