//-----------------------------------------------------------
/**
 * Classe ObjetFourmi
 *
 *
 * Copyright (C) 2004  Nicolas Luet + Laurent Couvidou
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Nicolas Luet + Laurent Couvidou
 * @date        juin 2004
 * @version     0.22
 */

   import java.awt.* ;

    public class ObjetFourmi extends ObjetVisible {
   
   // Etats primaires
      public static final int ETAT_AU_NID = 100 ;
      public static final int ETAT_RECHERCHE_NOURRITURE = 101 ;
      public static final int ETAT_COLLECTE_NOURRITURE = 102 ;
      public static final int ETAT_RAMENE_NOURRITURE = 103 ;
		public static final int ETAT_ATTENTE_INCUBATION = 104 ;
		public static final int ETAT_EN_INCUBATION = 105 ;
   
   // Etats Secondaires
      public static final int ETAT_CONTOURNEMENT_GAUCHE = 200 ;
      public static final int ETAT_CONTOURNEMENT_DROITE = 201 ;
      public static final int ETAT_TOUT_DROIT = 202 ;
      public static final int ETAT_RETOUR = 203 ;
      public static final int ETAT_SUIT_PHEROMONE = 204 ;
   
      protected int etatPrimaire, etatSecondaire, etatTernaire ;
      protected Simulation simu ;
      protected ObjetFourmiliere fourmiliere ;
      protected ObjetNourriture nourriture ;
      protected ObjetPheromone pheromone ;
   
      protected double vitesse = 2.0 ;
   
   // Attributs pour la nourriture
      protected int charge ;
   
   /** 
   * Constructeur 
   *
   * @param f                 fourmiliere de la fourmi
   * @param x0                coord x de la fourmi
   * @param y0                coord y de la fourmi
   * @param id0               numero de la fourmi 
   * @param idgrp0            numero de la fourmiliere (dfinit la couleur de la fourmi)
   */
   
       public ObjetFourmi(Simulation s, ObjetFourmiliere f, int x0, int y0, int id0, int idgrp0){
      
         super (x0, y0, id0, idgrp0) ;
         simu = s ;
         type = FOURMI ;
         fourmiliere = f ;
         setColor(Objet.RED[idgrp0], Objet.GREEN[idgrp0], Objet.BLUE[idgrp0]) ;
         roty= 180;
         scale = 0.3 ;
         etatPrimaire = ETAT_AU_NID ;
         etatSecondaire = 0 ;
      }
   
   /** 
   * Constructeur 
   *
   * @param f                 fourmiliere de la fourmi
   * @param x0                coord x de la fourmi
   * @param y0                coord y de la fourmi
   * @param sens              sens de la fourmi sur l'axe des y : -1 ou 1
   * @param id0               numero de la fourmi 
   * @param idgrp0            numero de la fourmiliere (dfinit la couleur de la fourmi)
   */
   
       public ObjetFourmi(Simulation s, ObjetFourmiliere f, int x0, int y0, int sens, int id0, int idgrp0){
      
         super (x0, y0, id0, idgrp0) ;
         simu = s ;
         type = FOURMI ;
         fourmiliere = f ;
         setColor(Objet.RED[idgrp0], Objet.GREEN[idgrp0], Objet.BLUE[idgrp0]) ;
         if (sens == -1)
            roty = 0;
         else
            roty = 180;
         scale = 0.3 ;
         etatPrimaire = ETAT_AU_NID ;
         etatSecondaire = 0 ;
      }
   
   //------------------------------------------------------------
   /** 
   * simulation de l'objet
   */
   
       public void run(){
		 }
		    
   //------------------------------------------------------------
   /** 
   * Dessin 2D de l'objet 
   */
       public void draw(Graphics g, double facteurZoom) {
         g.setColor(new Color (getRed(), getGreen(), getBlue()));
         g.fillRect((int)(x*facteurZoom), (int)(y*facteurZoom), 
               (int)(2*(facteurZoom+1)), (int)(2*(facteurZoom+1))) ;
      }
   }
