//------------------------------------------------------------
/**
 * SimAntGLAnimCanvas est la vue 3D pour la simulation d'une fourmiliere
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder
 * @date        janvier 2004
 * @version     0.22
 */

   import java.util.* ;
   import java.awt.*;
   import java.awt.event.*;

//bibliotheques implantation OpenGL pour le java : gl4java
   import gl4java.*;
   import gl4java.awt.*;

    class SimAntGLAnimCanvas extends GLAnimCanvas implements KeyListener, MouseMotionListener
   {
      private static final double CONV_DEG_RAD = Math.PI/180.0 ;
   
      private float largeur,profondeur ;
   
      private Simulation simu ;
   
   
   /** 
   * pour une gestion plus pousse de la souris
   */
      private int mousexold, mouseyold, mousex, mousey ;
   
      private TerrainGL terrainGL;
   
      private Camera camera;
   
      private boolean modeLighting, modeWireFrame, modeTexture ;
      private boolean modeFly ;
   
   //------------------------------------------------------------
   /** 
   * Constructeur 
   */
       public SimAntGLAnimCanvas(int w, int h, Simulation sim)
      {
         super(w, h);
      
         simu = sim ;
      
         modeLighting = true ;
         modeWireFrame = false ;
         modeTexture = true ;
      
         modeFly = false ;
      
      //on coute la souris et le clavier
         addKeyListener(this);
         addMouseListener(this);
         addMouseMotionListener (this);
      
      // animation affichant 20 images par seconde : valeur offrant un bon rapport performance graphique/qualit d'animation. Remarque : le nombre d'images par seconde affiches est indpendante des calculs de simulation.
         setAnimateFps(20); 
      
         requestFocus();
         start() ;
      }
   
   //------------------------------------------------------------
       public void setModeLighting(boolean b) {modeLighting=b;}
       public void setModeWireFrame(boolean b) {modeWireFrame=b;}
       public void setModeTexture(boolean b) {modeTexture=b;}
   
   //------------------------------------------------------------
       public boolean isModeLighting() {
         return modeLighting;}
       public boolean isModeWireFrame() {
         return modeWireFrame;}
       public boolean isModeTexture() {
         return modeTexture;}
   
   //------------------------------------------------------------
       public void toggleModeLighting() { modeLighting = !modeLighting ; }
       public void toggleModeWireFrame() { 
         modeWireFrame = !modeWireFrame ;
         modeTexture = !modeWireFrame ;
      }
       public void toggleModeTexture() { modeTexture = !modeTexture ; }
   
       public void toggleModeFly() { modeFly = !modeFly ; camera.initFly() ; }
   
   //------------------------------------------------------------
   /** 
   * Redimensionnement de la fenetre
   */
       public void reshape(int width, int height)
      {
      // Pour que le contexte occupe tout l'espace disponible
         gl.glViewport (0, 0, width,  height); //zone d'affichage de l'OpenGL
         gl.glMatrixMode (GL_PROJECTION); //Selection de la matrice de projection
         gl.glLoadIdentity (); //Initialisatin  l'identit
         glu.gluPerspective(60f, (double)width/(double) height, 1f, 10000.0f);
      
      // angle de vue de 60 degrs, vision profonde
         gl.glMatrixMode(GL_MODELVIEW); //retour  la matrice de modlisation
      }
   
   //------------------------------------------------------------
   /** 
   * Appel avant la creation du contexte
   */
       public void preInit() {
      // double buffering : rduit le scintillement.
         doubleBuffer = true; stereoView = false;
      }
   
   //------------------------------------------------------------
   /** 
   * Appel aprs la creation du contexte
   */
       public void init() {
         double width = (double)getSize().width;
         double height = (double)getSize().height;
      
         gl.glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
         gl.glEnable(GL_COLOR_MATERIAL);
      
      // glColor influe sur la couleur du materiau composante ambiante et diffusion
         gl.glEnable(GL_LIGHTING);
         gl.glEnable(GL_LIGHT0); // activation lumire standard
         gl.glCullFace(GL_BACK); // pas de calcul sur les surfaces interieures
         gl.glEnable(GL_CULL_FACE); // pas de calcul sur les surface caches
         gl.glShadeModel(GL_SMOOTH); // dgrad automatique des couleurs
      
      // Couleur pour effacer les buffers : noir
         gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
      
      // Rinitialisation du tampon de profondeur avec la valeur maximum
         gl.glClearDepth(1.0);
      
         gl.glEnable(GL_DEPTH_TEST); //test de profondeur activ
         gl.glEnable(GL_RESCALE_NORMAL);
      
         reshape((int)width,(int)height);
      
      // initialisation de la camra et du terrain
         camera=new Camera(simu.getLargeur(),simu.getProfondeur());
         terrainGL=new TerrainGL(this,(int)simu.getLargeur(),(int)simu.getProfondeur(),gl,glu);
      
      // chargement des objets ASE et cration des listes d'affichages
         Objet3DASE.init(gl) ;
      }
   
   //------------------------------------------------------------
   /** 
   * Dessin de la scene 3D
   */
       public void DrawGLScene() {
         gl.glEnable(GL_LIGHTING);
         gl.glPolygonMode(GL_FRONT,GL_FILL);
      
      // mode eclairage
         if(modeLighting) gl.glEnable(GL_LIGHTING);
         else gl.glDisable(GL_LIGHTING);
      
      // mode wireframe (fil de fer)
         if(modeWireFrame) gl.glPolygonMode(GL_FRONT,GL_LINE);
         else gl.glPolygonMode(GL_FRONT,GL_FILL);
      
      // Effacer les tampons
         gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      
      // Rinitialisation de la matrice de modlisation  l'identit
         gl.glLoadIdentity();
         gl.glPushMatrix();
      
      // Positionnement de la vue
         glu.gluLookAt((float)camera.getX(),
                  (float)camera.getY(),
                  (float)camera.getZ(),
                  (float)camera.getX2(),
                  (float)camera.getY2(),
                  (float)camera.getZ2(),0,1,0) ;
      
         gl.glPushMatrix();
      
      // Dessin des objets 3D
         int i = 0 ;
         int imax = simu.getNbrObjetsVisibles() ;
         while (i<imax) {
            Objet obj = simu.getObjetVisible(i) ;
            if (obj!=null) {
					if (((obj.getType()==2)&&(obj.getContenu()>0)) || (obj.getType()!=2)) {
                  if (!obj.isWithColor()) {
                     gl.glColor3f(obj.getRed(), obj.getGreen(), obj.getBlue());
                     gl.glDisable(gl.GL_TEXTURE_2D); // pour les fourmis
                  }
                  gl.glPushMatrix();
               
               // On inverse Y et Z car les coordonnes x, y de la simu
               // correspondent aux axes X et Z de la 3D (Y tant l'altitude).
                  gl.glTranslated(obj.getX(),obj.getZ(),obj.getY());
                  gl.glRotatef((float)obj.getRot(),0,1,0);
                  gl.glScaled(obj.getScale(),obj.getScale(),obj.getScale());
               
               // Appel de la liste d'affichage avec l'index correspondant
               // au type d'objet  afficher :
               // index d'indice 0 pour un obstacle (1)
               // index d'indice 1 pour la nourriture (2)
               // ...
                  gl.glCallList(Objet3DASE.getIndex(obj.getType()-1)); //appel de la liste d'affichage
               
                  gl.glPopMatrix();
               }
            }
            i++ ;
         }
         gl.glPopMatrix();
      
      // Dessin du terrain
         gl.glPushMatrix();
         terrainGL.draw(gl); 
         gl.glPopMatrix();
      
         gl.glPopMatrix();
      }
   
   //------------------------------------------------------------
   /** 
   * Gestion de l'affichage 
   */
       public void display() {
      // OpenGL correctement initialis ?
         if (!glj.gljMakeCurrent()) 
            return;
      
         if (modeFly) camera.fly() ;
         DrawGLScene();
      
      // Test des erreurs openGL
      // glj.gljCheckGL();
      
         glj.gljSwap(); // Swap des buffers
         glj.gljFree();
      }
   
   //------------------------------------------------------------
       public void keyTyped(KeyEvent e) {
      }
   
   //------------------------------------------------------------
   /** 
   * Ecouteur clavier
   */
       public void keyPressed(KeyEvent e) {
         switch(e.getKeyCode())
         {
            case KeyEvent.VK_ESCAPE: // Exit
               {
                  freeMem();
                  System.exit(0);
                  break;
               }
            case KeyEvent.VK_UP: // Deplacements de la camera        
               {
                  if (!modeFly) 
                     camera.translateDevant() ;
                  else 
                     camera.incRotXY (-Camera.VITESSE_ROTX,0.0) ;
                  break;
               }
            case KeyEvent.VK_DOWN:
               {
                  if (!modeFly) 
                     camera.translateDerriere() ;
                  else 
                     camera.incRotXY (Camera.VITESSE_ROTX,0.0) ;
                  break;
               }
            case KeyEvent.VK_RIGHT:
               {
                  if (!modeFly) 
                     camera.translateDroite() ;
                  else
                     camera.incRotXY(0.0,Camera.VITESSE_ROTY) ;
                  break;
               }
            case KeyEvent.VK_LEFT:
               {
                  if (!modeFly) 
                     camera.translateGauche() ;
                  else
                     camera.incRotXY (0.0,-Camera.VITESSE_ROTY) ;
                  break;
               }
            case KeyEvent.VK_GREATER : 
            case KeyEvent.VK_PLUS : 
            case 107 :
               {
                  if (modeFly)
                     camera.incVitesse() ;
                  break ;
               }
            case KeyEvent.VK_SUBTRACT : 
            case KeyEvent.VK_MINUS :
               {
                  if (modeFly)
                     camera.decVitesse() ;
                  break ;
               }
            case KeyEvent.VK_PAGE_DOWN: // variation d'altitude
               {
                  camera.translateBas() ;
                  break;
               }
            case KeyEvent.VK_PAGE_UP:
               {
                  camera.translateHaut() ;
                  break;
               }
            case KeyEvent.VK_W: // Mode wireframe
               {
                  toggleModeWireFrame() ;
                  break;
               }
            case KeyEvent.VK_L: // Mode lighting
               {
                  toggleModeLighting() ;
                  break;
               }
            case KeyEvent.VK_T: // Mode texture
               {
                  toggleModeTexture() ;
                  break;
               }
            case KeyEvent.VK_F: // Mode fly
               {
                  toggleModeFly() ;
                  break;
               }
            case KeyEvent.VK_R: // Mode Reinit
               {
                  simu.stop() ;
                  simu.reinit() ;
                  simu.start() ;
                  break;
               }
         
         }
      }
   
   //------------------------------------------------------------
       public void keyReleased(KeyEvent e) {
      }
   
   //------------------------------------------------------------
       public void mouseEntered(MouseEvent evt) {
         Component comp = evt.getComponent();
         if( comp.equals(this) ) {
            requestFocus();
         }
      }
   
   //------------------------------------------------------------
   /** 
   * Ecouteur souris
   */
       public void mouseDragged(MouseEvent evt){
         mousex=evt.getX();
         mousey=evt.getY();
      
      // Rotations ft des dplacement de souris
         camera.incRotXY((mousey-mouseyold)/10,(mousex-mousexold)/10) ;
      
         mousexold=mousex;
         mouseyold=mousey;
      }
   
   //------------------------------------------------------------
       public void mouseMoved(MouseEvent evt){
      // mmorisation des coord pour justesse des calculs  la reprise du drag 
         mousex=evt.getX();
         mousey=evt.getY();
      
         mousexold=mousex;
         mouseyold=mousey;
      }
   
   //------------------------------------------------------------
       public void mouseExited(MouseEvent evt){
      }
   
   //------------------------------------------------------------
       public void mousePressed(MouseEvent evt){
      }
   
   //------------------------------------------------------------
       public void mouseReleased(MouseEvent evt){
      }
   
   //------------------------------------------------------------
       public void mouseClicked(MouseEvent evt)
      {
         Component comp = evt.getComponent();
         if( comp.equals(this) ) {
            requestFocus();
         }
      }
   
   //------------------------------------------------------------
   /** 
   * Liberation de la mmoire graphique : suppression des listes d'affichage
   */
       public void freeMem(){  
      }
   }

