//------------------------------------------------------------
/**
 * Simulation
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder 
 * @date        janvier 2004
 * @version     0.22
 */

   import java.util.* ;

    public class Simulation implements Runnable
   {
   
      public static final double CONV_DEG_RAD = Math.PI/180.0 ;
   
      public static final int MAX_FOURMILIERE = 7 ;
   
      //public static final int NBR_OBSTACLES_INIT = 30 ; 
      //public static final int NBR_NOURRITURES_INIT = 50 ;
      //public static final int NBR_FOURMILIERES_INIT = 1 ;
      //public static final int nbrFourmiInitial = 10 ; // par fourmiliere
   	
      public static final int OBSTACLE = 1 ;
      public static final int NOURRITURE = 2 ;
      public static final int FOURMILIERE = 3 ;
      public static final int FOURMI = 4 ;
      public static final int PHEROMONE = 5 ;
   
      protected float largeur,profondeur ;
   
      private int nbrObstacles  ; 
      private int nbrNourritures ;
      private int nbrFourmilieres ;
      private int nbrFourmis;
   
      private Terrain terrain ;
   
      public Vector listeObjetsVisibles ;
      public Vector listeObjetsInvisibles ;
   
      private boolean finished ;
      private boolean exitRun=false ;
   
      private int attenteSimu=40 ;
   
   //------------------------------------------------------------
   /** 
   * Constructeur par dfaut
   */
       public Simulation(int l, int p, int nfl, int nfm, int nn, int no) {
         
         largeur = l ;
         profondeur = p ;
         nbrObstacles = no ;
         nbrNourritures = nn ;
         nbrFourmilieres = nfl ;
         nbrFourmis = nfm ;
         genererObjets() ;
      }
   
   //------------------------------------------------------------
       public float getLargeur() { 
         return largeur ;}
       public float getProfondeur() {
         return profondeur ;}
   
   //------------------------------------------------------------
   /** 
   * Reinit
   */
       public void reinit() {
      //    largeur = profondeur = 2000 ;
      //   nbrObstacles = no ;
      //   nbrNourritures = nn ;
      //   nbrFourmilieres = nfl ;
      //  nbrFourmis = nfm ;
         genererObjets() ;
      }
   
   //------------------------------------------------------------
   /** 
   * Generation de l'environnement
   */
       public void genererObjets() {
         listeObjetsVisibles = new Vector() ;
         listeObjetsInvisibles = new Vector() ;
      	
      	/* Gnration des obstacles */
         for (int i=0;i<nbrObstacles;i++) {
            int x = (int)(Math.random()*largeur) ;
            int y = (int)(Math.random()*profondeur) ;
            float ry=(float)(Math.random()*360.0);
            float s=(float)(0.5+Math.random()*2.0);
         // NL+LC
            if (i==0) {
               x = 1200 ; y =1000 ;
               s = (float)2.0 ;
            }
            listeObjetsVisibles.add (new ObjetObstacle(x, y, ry, s, i,0)) ;
         }
      	
      	/* Gnration de la nourriture */
         for (int i=0;i<nbrNourritures;i++) {
            int x = (int)(Math.random()*largeur) ;
            int y = (int)(Math.random()*profondeur) ;
            float ry=(float)(Math.random()*360.0);
            float s=(float)(0.5+Math.random()*0.5);
            if (i==0) {
               x =1000; y = 1200 ;
               s = (float)2.0 ;
            }
            listeObjetsVisibles.add (new ObjetNourriture(x, y, ry, s, i,0)) ;
         }
      	
      	/* Gnration des fourmilires */
         for (int i=0;i<nbrFourmilieres;i++) {
            int x = (int)(Math.random()*largeur) ;
            int y = (int)(Math.random()*profondeur) ;
            float ry=(float)(Math.random()*360.0);
            int pop = nbrFourmis;
            if (i==0) {
            // 3D
               x = 1000; y =1000 ;
               ry = 180 ;
            }
            ObjetFourmiliere of = new ObjetFourmiliere(x, y, ry, i, pop) ;
            listeObjetsVisibles.add (of) ;
         	
         	/* Gnration de la reine */
            ObjetFourmiReine r = new ObjetFourmiReine(this, of, x, y, 0, i);
            listeObjetsVisibles.add (r);
         	
         	/* Gnration des ouvrires */
            for (int j=1;j<=nbrFourmis;j++) {
               ObjetFourmiOuvriere f = new ObjetFourmiOuvriere(this, of, x, y, j, i) ;
               listeObjetsVisibles.add (f) ;
            }
         }
      }
   
   //------------------------------------------------------------
   /** 
   * Detection d'obstacle
   */
       public Objet isObstacle(double x, double y) {
         int i = 0 ;
         int imax = getNbrObjetsVisibles() ;
         while (i<imax) {
            Objet obj = getObjetVisible(i) ;
            if ((obj!=null) && (obj.contains(x,y)) && ((obj.getType() == 1)||(obj.getType() == 2))) {
               return obj ;
            }
            i++ ;
         }
         return null ;
      }
   
   //------------------------------------------------------------
   /** 
   * Detection de nourriture
   */
       public Objet isNourriture(double x, double y) {
         int i = 0 ;
         int imax = getNbrObjetsVisibles() ;
         while (i<imax) {
            Objet obj = getObjetVisible(i) ;
            if ((obj!=null) && (obj.contains(x,y)) && (obj.getType() == NOURRITURE)) {
               return obj ;
            }
            i++ ;
         }
         return null ;
      }
   	
   /* Suppression de nourriture */
   
       public synchronized void suppNourriture(ObjetNourriture obj) {
         //synchronized (obj){
         if (listeObjetsVisibles.indexOf (obj)!=-1){
            listeObjetsVisibles.remove(listeObjetsVisibles.indexOf (obj));
         }
         //}
      }
   	
   /* Poser une phromone */
   
       public void poserPheromone(ObjetFourmi of, ObjetNourriture on) {
         int x = (int)of.getX();
         int y = (int)of.getY();
         int id = of.getId();
         int idg = of.getIdGroup();
         double xn = on.getX();
         double yn = on.getY();
         ObjetPheromone op = new ObjetPheromone(x, y, id, idg, xn, yn) ;
         listeObjetsInvisibles.add (op) ;
      }
   	
   	
   /* Dtecter une phromone */
   
       public Objet isPheromone(double x, double y) {
         int i = 0 ;
         int imax = getNbrObjetsInvisibles() ;
         while (i<imax) {
            Objet obj = getObjetInvisible(i) ;
            if ((obj!=null) && (Math.abs(x-obj.getX())<2.0) && (Math.abs(y-obj.getY())<2.0) && (obj.getType() == PHEROMONE)) {
               return obj ;
            }
            i++ ;
         }
         return null ;
      }
   	
   /* Crer une fourmi */
   
       public void creerFourmi (ObjetFourmiliere of, int x, int y, int id, int idg) {
         ObjetFourmiOuvriere ofo = new ObjetFourmiOuvriere(this, of, x, y, id, idg) ;
         listeObjetsVisibles.add (ofo) ;
      }
   	
   	
   	
   //------------------------------------------------------------
   /** 
   * init de la simu
   */
       public void start() {
         Thread thread = new Thread(this) ;
         finished = false; 
         exitRun = false ;
         thread.start() ;
      }
   
   //------------------------------------------------------------
   /** 
   * fin de la simu
   */
       public void stop() {
         finished = true ;
         while (!exitRun) {
            try{ Thread.currentThread().sleep(attenteSimu); } 
                catch(Exception e) {}
         }
      }
   
   //------------------------------------------------------------
   /** 
   * run du thread
   */
       public void run() {
         exitRun = false ;
         while (!finished) {
            int i = 0 ;
            int imax = getNbrObjetsVisibles() ;
            while (i<imax) {
               Objet obj = getObjetVisible(i) ;
               if (obj!=null) {
                  obj.run() ;
               }
               i++ ;
            }
         
         // Pause dans la simulation
            try{ Thread.currentThread().sleep(attenteSimu); } 
                catch(Exception e) {}
         }
         exitRun = true ;
      }
   
   //------------------------------------------------------------
   /** 
   * renvoie un iterateur
   */
       public Iterateur getIterateur() { 
         return new Iterateur() ;}
   
   //------------------------------------------------------------
   /** 
   * Classe interne Iterateur
   */
       public class Iterateur {
         private int index=-1 ;
          public Iterateur() {}
          public boolean hasNext() { 
            return (index+1)<listeObjetsVisibles.size() ;}
          public Objet getNext() { 
            return (Objet)listeObjetsVisibles.get(++index) ;
         }
      }
   
   //------------------------------------------------------------
   /** 
   * renvoie les objets
   */
       public int getNbrObjetsVisibles() { 
         return listeObjetsVisibles.size() ;
      }
   	
       public int getNbrObjetsInvisibles() { 
         return listeObjetsInvisibles.size() ;
      }
   
       public ObjetVisible getObjetVisible(int i) { 
         return (ObjetVisible)listeObjetsVisibles.get(i) ;
      }
   	
       public ObjetInvisible getObjetInvisible(int i) { 
         return (ObjetInvisible)listeObjetsInvisibles.get(i) ;
      }
   	
       public ObjetNourriture getObjetNourriture(int i) { 
         return (ObjetNourriture)listeObjetsVisibles.get(i) ;
      }
   
   }
